@@ -15,7 +15,7 @@ from pymavlink.dialects.v10 import ardupilotmega
 
 # these imports allow for mavgraph and mavlogdump to use maths expressions more easily
 from math import *
-from mavextra import *
+from .mavextra import *
 
 '''
 Support having a $HOME/.pymavlink/mavextra.py for extra graphing functions
@@ -71,7 +71,7 @@ def set_dialect(dialect):
     For example, set_dialect("ardupilotmega")
     '''
     global mavlink, current_dialect
-    from generator import mavparse
+    from .generator import mavparse
     if mavlink is None or mavlink.WIRE_PROTOCOL_VERSION == "1.0" or not 'MAVLINK09' in os.environ:
         wire_protocol = mavparse.PROTOCOL_1_0
         modname = "pymavlink.dialects.v10." + dialect
@@ -125,6 +125,8 @@ class mavfile(object):
         self.param_fetch_complete = False
         self.start_time = time.time()
         self.flightmode = "UNKNOWN"
+        self.vehicle_type = "UNKNOWN"
+        self.mav_type = mavlink.MAV_TYPE_FIXED_WING
         self.base_mode = 0
         self.timestamp = 0
         self.message_hooks = []
@@ -234,6 +236,7 @@ class mavfile(object):
             self.target_component = msg.get_srcComponent()
             if mavlink.WIRE_PROTOCOL_VERSION == '1.0' and msg.type != mavlink.MAV_TYPE_GCS:
                 self.flightmode = mode_string_v10(msg)
+                self.mav_type = msg.type
                 self.base_mode = msg.base_mode
         elif type == 'PARAM_VALUE':
             s = str(msg.param_id)
@@ -435,11 +438,15 @@ class mavfile(object):
 
     def mode_mapping(self):
         '''return dictionary mapping mode names to numbers, or None if unknown'''
-        mav_type = self.field('HEARTBEAT', 'type', None)
+        mav_type = self.field('HEARTBEAT', 'type', self.mav_type)
         if mav_type is None:
             return None
         map = None
-        if mav_type == mavlink.MAV_TYPE_QUADROTOR:
+        if mav_type in [mavlink.MAV_TYPE_QUADROTOR,
+                        mavlink.MAV_TYPE_HELICOPTER,
+                        mavlink.MAV_TYPE_HEXAROTOR,
+                        mavlink.MAV_TYPE_OCTOROTOR,
+                        mavlink.MAV_TYPE_TRICOPTER]:
             map = mode_mapping_acm
         if mav_type == mavlink.MAV_TYPE_FIXED_WING:
             map = mode_mapping_apm
@@ -665,13 +672,17 @@ class mavserial(mavfile):
         self.baud = baud
         self.device = device
         self.autoreconnect = autoreconnect
-        self.port = serial.Serial(self.device, self.baud, timeout=0,
+        # we rather strangely set the baudrate initially to 1200, then change to the desired
+        # baudrate. This works around a kernel bug on some Linux kernels where the baudrate
+        # is not set correctly
+        self.port = serial.Serial(self.device, 1200, timeout=0,
                                   dsrdtr=False, rtscts=False, xonxoff=False)
         try:
             fd = self.port.fileno()
             set_close_on_exec(fd)
         except Exception:
             fd = None
+        self.set_baudrate(self.baud)
         mavfile.__init__(self, fd, device, source_system=source_system)
         self.rtscts = False
 
@@ -679,6 +690,10 @@ class mavserial(mavfile):
         '''enable/disable RTS/CTS if applicable'''
         self.port.setRtsCts(enable)
         self.rtscts = enable
+
+    def set_baudrate(self, baudrate):
+        '''set baudrate'''
+        self.port.setBaudrate(baudrate)
     
     def close(self):
         self.port.close()
@@ -715,7 +730,7 @@ class mavserial(mavfile):
                 except Exception:
                     self.fd = None
                 if self.rtscts:
-                    set_rtscts(self.rtscts)
+                    self.set_rtscts(self.rtscts)
                 return
             except Exception:
                 print("Failed to reopen %s" % self.device)
@@ -1159,6 +1174,7 @@ mode_mapping_apm = {
     5 : 'FBWA',
     6 : 'FBWB',
     7 : 'CRUISE',
+    8 : 'AUTOTUNE',
     10 : 'AUTO',
     11 : 'RTL',
     12 : 'LOITER',
@@ -1199,6 +1215,54 @@ mode_mapping_tracker = {
     16 : 'INITIALISING'
     }
 
+mode_mapping_px4 = {
+    0 : 'MANUAL',
+    1 : 'ATTITUDE',
+    2 : 'EASY',
+    3 : 'AUTO'
+    }
+
+
+def mode_mapping_byname(mav_type):
+    '''return dictionary mapping mode names to numbers, or None if unknown'''
+    map = None
+    if mav_type in [mavlink.MAV_TYPE_QUADROTOR,
+                    mavlink.MAV_TYPE_HELICOPTER,
+                    mavlink.MAV_TYPE_HEXAROTOR,
+                    mavlink.MAV_TYPE_OCTOROTOR,
+                    mavlink.MAV_TYPE_TRICOPTER]:
+        map = mode_mapping_acm
+    if mav_type == mavlink.MAV_TYPE_FIXED_WING:
+        map = mode_mapping_apm
+    if mav_type == mavlink.MAV_TYPE_GROUND_ROVER:
+        map = mode_mapping_rover
+    if mav_type == mavlink.MAV_TYPE_ANTENNA_TRACKER:
+        map = mode_mapping_tracker
+    if map is None:
+        return None
+    inv_map = dict((a, b) for (b, a) in map.items())
+    return inv_map
+
+def mode_mapping_bynumber(mav_type):
+    '''return dictionary mapping mode numbers to name, or None if unknown'''
+    map = None
+    if mav_type in [mavlink.MAV_TYPE_QUADROTOR,
+                    mavlink.MAV_TYPE_HELICOPTER,
+                    mavlink.MAV_TYPE_HEXAROTOR,
+                    mavlink.MAV_TYPE_OCTOROTOR,
+                    mavlink.MAV_TYPE_TRICOPTER]:
+        map = mode_mapping_acm
+    if mav_type == mavlink.MAV_TYPE_FIXED_WING:
+        map = mode_mapping_apm
+    if mav_type == mavlink.MAV_TYPE_GROUND_ROVER:
+        map = mode_mapping_rover
+    if mav_type == mavlink.MAV_TYPE_ANTENNA_TRACKER:
+        map = mode_mapping_tracker
+    if map is None:
+        return None
+    return map
+
+
 def mode_string_v10(msg):
     '''mode string for 1.0 protocol, from heartbeat'''
     if not msg.base_mode & mavlink.MAV_MODE_FLAG_CUSTOM_MODE_ENABLED:
@@ -1228,7 +1292,12 @@ def mode_string_acm(mode_number):
     if mode_number in mode_mapping_acm:
         return mode_mapping_acm[mode_number]
     return "Mode(%u)" % mode_number
-    
+
+def mode_string_px4(mode_number):
+    '''return mode string for PX4 flight stack'''
+    if mode_number in mode_mapping_px4:
+        return mode_mapping_px4[mode_number]
+    return "Mode(%u)" % mode_number
 
 class x25crc(object):
     '''x25 CRC - based on checksum.h from mavlink library'''
@@ -1283,13 +1352,15 @@ class MavlinkSerialPort():
                         n = len(b)
                         if n > 70:
                                 n = 70
+                        buf = [ord(x) for x in b[:n]]
+                        buf.extend([0]*(70-len(buf)))
                         self.mav.mav.serial_control_send(self.port,
                                                          mavutil.mavlink.SERIAL_CONTROL_FLAG_EXCLUSIVE |
                                                          mavutil.mavlink.SERIAL_CONTROL_FLAG_RESPOND,
                                                          0,
                                                          0,
                                                          n,
-                                                         b[:n])
+                                                         buf)
                         b = b[n:]
 
         def _recv(self):
@@ -1306,7 +1377,7 @@ class MavlinkSerialPort():
                                                          mavutil.mavlink.SERIAL_CONTROL_FLAG_RESPOND,
                                                          0,
                                                          0,
-                                                         0, '')
+                                                         0, [0]*70)
                         m = self.mav.recv_match(condition='SERIAL_CONTROL.count!=0',
                                                 type='SERIAL_CONTROL', blocking=True, timeout=0.01)
                         if m is not None and m.count != 0:
@@ -1315,7 +1386,8 @@ class MavlinkSerialPort():
                 if m is not None:
                         if self._debug > 2:
                                 print(m)
-                        self.buf += str(m.data[:m.count])
+                        data = m.data[:m.count]
+                        self.buf += ''.join(str(chr(x)) for x in data)
 
         def read(self, n):
                 '''read some bytes'''
@@ -1352,6 +1424,6 @@ class MavlinkSerialPort():
                                                  mavutil.mavlink.SERIAL_CONTROL_FLAG_EXCLUSIVE,
                                                  0,
                                                  self.baudrate,
-                                                 0, '')
+                                                 0, [0]*70)
                 self.flushInput()
                 self.debug("Changed baudrate %u" % self.baudrate)
